/*
 * Copyright 2011 Vaadin Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.vaadin.terminal.gwt.client.ui;

/**
 * Widgets who wish to be notificed when a shortcut action has been triggered
 * with the widget as a target should implement this interface. The
 * {@link #handleAction(ShortcutAction)} method will be called just before the
 * action is communicated to the server
 * 
 */
public interface ShortcutActionTarget {

    /**
     * Called by the {@link ShortcutActionHandler} just before the shortcut
     * action is sent to the server side
     * 
     * @param action
     *            The action which will be performed on the server side
     * @return Returns true if the shortcut was handled
     */
    boolean handleAction(ShortcutAction action);
}
